@@ -3,7 +3,7 @@
 
 import abc
 
-from pyhocon import ConfigTree  # noqa: F401
+from pyhocon import ConfigTree
 
 from databuilder import Scoped
 
@@ -13,19 +13,16 @@ class Task(Scoped):
     A Abstract task that can run an abstract task
     """
     @abc.abstractmethod
-    def init(self, conf):
-        # type: (ConfigTree) -> None
+    def init(self, conf: ConfigTree) -> None:
         pass
 
     @abc.abstractmethod
-    def run(self):
-        # type: () -> None
+    def run(self) -> None:
         """
         Runs a task
         :return:
         """
         pass
 
-    def get_scope(self):
-        # type: () -> str
+    def get_scope(self) -> str:
         return 'task'
